@@ -28,6 +28,10 @@ Copyright_License {
 #include <assert.h>
 #include <string.h>
 
+#ifdef __arm__
+
+bool is_nook, is_dithered;
+
 bool
 IsGalaxyTab22()
 {
@@ -38,18 +42,4 @@ IsGalaxyTab22()
      strcmp(native_view->GetProduct(), "GT-P1010") == 0);
 }
 
-bool
-IsNookSimpleTouch()
-{
-  if (native_view == nullptr)
-    return false;
-
-  static int cached_type = -1;
-  if (cached_type == -1) {
-    if (strcmp(native_view->GetProduct(), "NOOK") == 0)
-      cached_type = 1;
-    else
-      cached_type = 0;
-  }
-  return cached_type == 1;
-}
+#endif
