@@ -30,9 +30,9 @@
 #include "Language/Language.hpp"
 #include "Form/DataField/Enum.hpp"
 #include "Form/DataField/Boolean.hpp"
+#include "Form/DataField/String.hpp"
 #include "Device/Register.hpp"
 #include "Device/Driver.hpp"
-#include "Profile/DeviceConfig.hpp"
 #include "Blackboard/DeviceBlackboard.hpp"
 #include "Interface.hpp"
 
@@ -51,12 +51,12 @@
 #endif
 
 enum ControlIndex {
-  Port, BaudRate, BulkBaudRate, TCPPort, I2CBus, I2CAddr, PressureUsage, Driver,
+  Port, BaudRate, BulkBaudRate,
+  IP_ADDRESS,
+  TCPPort,
+  I2CBus, I2CAddr, PressureUsage, Driver,
   SyncFromDevice, SyncToDevice,
   K6Bt,
-#ifndef NDEBUG
-  DumpPort,
-#endif
   IgnoreCheckSum,
 };
 
@@ -79,6 +79,8 @@ static constexpr struct {
   { DeviceConfig::PortType::IOIOVOLTAGE, N_("IOIO voltage sensor") },
 #endif
 
+  { DeviceConfig::PortType::TCP_CLIENT, N_("TCP client") },
+
   /* label not translated for now, until we have a TCP/UDP port
      selection UI */
   { DeviceConfig::PortType::TCP_LISTENER, N_("TCP Port") },
@@ -120,7 +122,11 @@ DetectSerialPorts(DataFieldEnum &df)
   bool found = false;
   const char *path;
   while ((path = enumerator.Next()) != nullptr) {
-    AddPort(df, DeviceConfig::PortType::SERIAL, path);
+    const char *display_string = path;
+    if (memcmp(path, "/dev/", 5) == 0)
+      display_string = path + 5;
+
+    AddPort(df, DeviceConfig::PortType::SERIAL, path, display_string);
     found = true;
   }
 
@@ -307,6 +313,7 @@ FillTCPPorts(DataFieldEnum &dfe)
 {
   dfe.addEnumText(_T("4353"), 4353);
   dfe.addEnumText(_T("10110"), 10110);
+  dfe.addEnumText(_T("4352"), 4352);
 }
 
 static void
@@ -349,6 +356,7 @@ SetPort(DataFieldEnum &df, const DeviceConfig &config)
   case DeviceConfig::PortType::NUNCHUCK:
   case DeviceConfig::PortType::I2CPRESSURESENSOR:
   case DeviceConfig::PortType::IOIOVOLTAGE:
+  case DeviceConfig::PortType::TCP_CLIENT:
   case DeviceConfig::PortType::TCP_LISTENER:
   case DeviceConfig::PortType::UDP_LISTENER:
   case DeviceConfig::PortType::PTY:
@@ -388,6 +396,11 @@ DeviceEditWidget::SetConfig(const DeviceConfig &_config)
 {
   config = _config;
 
+  if (config.port_type == DeviceConfig::PortType::DISABLED)
+    /* if the user configures a new device, forget the old "enabled"
+       flag and re-enable the device */
+    config.enabled = true;
+
   WndProperty &port_control = GetControl(Port);
   DataFieldEnum &port_df = *(DataFieldEnum *)port_control.GetDataField();
   SetPort(port_df, config);
@@ -404,6 +417,12 @@ DeviceEditWidget::SetConfig(const DeviceConfig &_config)
   bulk_baud_df.Set(config.bulk_baud_rate);
   bulk_baud_control.RefreshDisplay();
 
+  WndProperty &ip_address_control = GetControl(IP_ADDRESS);
+  DataFieldEnum &ip_address_df = *(DataFieldEnum *)
+    ip_address_control.GetDataField();
+  ip_address_df.Set(config.ip_address);
+  ip_address_control.RefreshDisplay();
+
   WndProperty &tcp_port_control = GetControl(TCPPort);
   DataFieldEnum &tcp_port_df = *(DataFieldEnum *)
     tcp_port_control.GetDataField();
@@ -451,16 +470,6 @@ DeviceEditWidget::SetConfig(const DeviceConfig &_config)
   k6bt_df.Set(config.k6bt);
   k6bt_control.RefreshDisplay();
 
-#ifndef NDEBUG
-
-  WndProperty &dump_port_control = GetControl(DumpPort);
-  DataFieldBoolean &dump_port_df =
-      *(DataFieldBoolean *)dump_port_control.GetDataField();
-  dump_port_df.Set(config.dump_port);
-  dump_port_control.RefreshDisplay();
-
-#endif
-
   WndProperty &ignore_checksum_control = GetControl(IgnoreCheckSum);
   DataFieldBoolean &ignore_checksum_df =
       *(DataFieldBoolean *)ignore_checksum_control.GetDataField();
@@ -543,6 +552,7 @@ DeviceEditWidget::UpdateVisibilities()
   SetRowVisible(BulkBaudRate, uses_speed &&
                 DeviceConfig::UsesDriver(type) &&
                 SupportsBulkBaudRate(GetDataField(Driver)));
+  SetRowAvailable(IP_ADDRESS, DeviceConfig::UsesIPAddress(type));
   SetRowAvailable(TCPPort, DeviceConfig::UsesTCPPort(type));
   SetRowAvailable(I2CBus, DeviceConfig::UsesI2C(type));
   SetRowAvailable(I2CAddr, DeviceConfig::UsesI2C(type) &&
@@ -555,10 +565,6 @@ DeviceEditWidget::UpdateVisibilities()
                 CanSendSettings(GetDataField(Driver)));
   SetRowAvailable(K6Bt, maybe_bluetooth);
 
-#ifndef NDEBUG
-  SetRowVisible(DumpPort, DeviceConfig::UsesPort(type));
-#endif
-
   SetRowVisible(IgnoreCheckSum, DeviceConfig::UsesDriver(type));
 }
 
@@ -567,19 +573,16 @@ DeviceEditWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
 {
   RowFormWidget::Prepare(parent, rc);
 
-  DataFieldEnum *port_df = new DataFieldEnum(NULL);
-  port_df->SetListener(this);
+  DataFieldEnum *port_df = new DataFieldEnum(this);
   FillPorts(*port_df, config);
   Add(_("Port"), NULL, port_df);
 
-  DataFieldEnum *baud_rate_df = new DataFieldEnum(NULL);
-  baud_rate_df->SetListener(this);
+  DataFieldEnum *baud_rate_df = new DataFieldEnum(this);
   FillBaudRates(*baud_rate_df);
   baud_rate_df->Set(config.baud_rate);
   Add(_("Baud rate"), NULL, baud_rate_df);
 
-  DataFieldEnum *bulk_baud_rate_df = new DataFieldEnum(NULL);
-  bulk_baud_rate_df->SetListener(this);
+  DataFieldEnum *bulk_baud_rate_df = new DataFieldEnum(this);
   bulk_baud_rate_df->addEnumText(_T("Default"), 0u);
   FillBaudRates(*bulk_baud_rate_df);
   bulk_baud_rate_df->Set(config.bulk_baud_rate);
@@ -587,21 +590,22 @@ DeviceEditWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
       _("The baud rate used for bulk transfers, such as task declaration or flight download."),
       bulk_baud_rate_df);
 
-  DataFieldEnum *tcp_port_df = new DataFieldEnum(NULL);
-  tcp_port_df->SetListener(this);
+  DataFieldString *ip_address_df = new DataFieldString(_T(""), this);
+  ip_address_df->Set(config.ip_address);
+  Add(_("IP Address"), NULL, ip_address_df);
+
+  DataFieldEnum *tcp_port_df = new DataFieldEnum(this);
   FillTCPPorts(*tcp_port_df);
   tcp_port_df->Set(config.tcp_port);
   Add(_("TCP Port"), NULL, tcp_port_df);
 
-  DataFieldEnum *i2c_bus_df = new DataFieldEnum(NULL);
-  i2c_bus_df->SetListener(this);
+  DataFieldEnum *i2c_bus_df = new DataFieldEnum(this);
   FillI2CBus(*i2c_bus_df);
   i2c_bus_df->Set(config.i2c_bus);
   Add(_("I2C Bus"), _("Select the description or bus number that matches your configuration."),
                       i2c_bus_df);
 
-  DataFieldEnum *i2c_addr_df = new DataFieldEnum(NULL);
-  i2c_addr_df->SetListener(this);
+  DataFieldEnum *i2c_addr_df = new DataFieldEnum(this);
   FillI2CAddr(*i2c_addr_df);
   i2c_addr_df->Set(config.i2c_addr);
   Add(_("I2C Addr"), _("The i2c address that matches your configuration."
@@ -609,8 +613,7 @@ DeviceEditWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
                         "In case you do not understand the previous sentence you may assume that this field is not used."),
                         i2c_addr_df);
 
-  DataFieldEnum *press_df = new DataFieldEnum(NULL);
-  press_df->SetListener(this);
+  DataFieldEnum *press_df = new DataFieldEnum(this);
   FillPress(*press_df);
   press_df->Set((unsigned)config.press_use);
   Add(_("Pressure use"), _("Select the purpose of this pressure sensor. "
@@ -618,8 +621,7 @@ DeviceEditWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
                            "what pressure this is and what its should be used for."),
                            press_df);
 
-  DataFieldEnum *driver_df = new DataFieldEnum(NULL);
-  driver_df->SetListener(this);
+  DataFieldEnum *driver_df = new DataFieldEnum(this);
 
   const struct DeviceRegister *driver;
   for (unsigned i = 0; (driver = GetDriverByIndex(i)) != NULL; i++)
@@ -647,16 +649,6 @@ DeviceEditWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
              config.k6bt, this);
   SetExpertRow(K6Bt);
 
-#ifndef NDEBUG
-
-  AddBoolean(_T("DumpPort"),
-             _("Enable this if you would like to log the communication with "
-               "the device."),
-             config.dump_port, this);
-  SetExpertRow(DumpPort);
-
-#endif
-
   AddBoolean(_("Ignore checksum"),
              _("If your GPS device outputs invalid NMEA checksums, this will "
                "allow it's data to be used anyway."),
@@ -687,6 +679,7 @@ FinishPortField(DeviceConfig &config, const DataFieldEnum &df)
   case DeviceConfig::PortType::NUNCHUCK:
   case DeviceConfig::PortType::I2CPRESSURESENSOR:
   case DeviceConfig::PortType::IOIOVOLTAGE:
+  case DeviceConfig::PortType::TCP_CLIENT:
   case DeviceConfig::PortType::TCP_LISTENER:
   case DeviceConfig::PortType::UDP_LISTENER:
   case DeviceConfig::PortType::RFCOMM_SERVER:
@@ -748,6 +741,10 @@ DeviceEditWidget::Save(bool &_changed)
     changed |= SaveValue(BulkBaudRate, config.bulk_baud_rate);
   }
 
+  if (config.UsesIPAddress())
+    changed |= SaveValue(IP_ADDRESS, config.ip_address.buffer(),
+                         config.ip_address.MAX_SIZE);
+
   if (config.UsesTCPPort())
     changed |= SaveValue(TCPPort, config.tcp_port);
 
@@ -773,11 +770,6 @@ DeviceEditWidget::Save(bool &_changed)
   if (CommonInterface::Basic().sensor_calibration_available)
     changed = true;
 
-#ifndef NDEBUG
-  if (config.UsesPort())
-    changed |= SaveValue(DumpPort, config.dump_port);
-#endif
-
   _changed |= changed;
   return true;
 }
