@@ -52,7 +52,7 @@ Copyright_License {
  * @return True if FAI shape
  */
 static bool
-TaskSummaryShape(OrderedTask* task, TCHAR* text)
+TaskSummaryShape(const OrderedTask *task, TCHAR *text)
 {
   bool FAIShape = false;
   switch (task->TaskSize()) {
@@ -97,7 +97,7 @@ TaskSummaryShape(OrderedTask* task, TCHAR* text)
   return FAIShape;
 }
 void
-OrderedTaskSummary(OrderedTask* task, TCHAR* text, bool linebreaks)
+OrderedTaskSummary(const OrderedTask *task, TCHAR *text, bool linebreaks)
 {
   const TaskStats &stats = task->GetStats();
   TCHAR summary_shape[100];
@@ -242,7 +242,7 @@ OrderedTaskPointRadiusLabel(const ObservationZonePoint &ozp, TCHAR* buffer)
 bool
 OrderedTaskSave(const OrderedTask &task)
 {
-  assert(protected_task_manager != NULL);
+  assert(protected_task_manager != nullptr);
 
   TCHAR fname[69] = _T("");
   if (!TextEntryDialog(fname, 64, _("Enter a task name")))
