@@ -25,20 +25,11 @@ Copyright_License {
 #include "Form/Internal.hpp"
 #include "Look/DialogLook.hpp"
 #include "DataField/Base.hpp"
-#include "DataField/String.hpp"
-#include "DataField/Prefix.hpp"
-#include "DataField/RoughTime.hpp"
-#include "DataField/GeoPoint.hpp"
 #include "Screen/Canvas.hpp"
-#include "Screen/Bitmap.hpp"
 #include "Screen/Layout.hpp"
 #include "Screen/Key.h"
 #include "Screen/Features.hpp"
-#include "Dialogs/ComboPicker.hpp"
-#include "Dialogs/TextEntry.hpp"
-#include "Dialogs/TimeEntry.hpp"
-#include "Dialogs/GeoPointEntry.hpp"
-#include "resource.h"
+#include "Dialogs/DataField.hpp"
 
 #include <assert.h>
 
@@ -159,7 +150,7 @@ WndProperty::~WndProperty()
 UPixelScalar
 WndProperty::GetRecommendedCaptionWidth() const
 {
-  return look.text_font->TextSize(caption).cx + Layout::FastScale(3);
+  return look.text_font->TextSize(caption).cx + Layout::GetTextPadding();
 }
 
 void
@@ -179,51 +170,15 @@ WndProperty::BeginEditing()
     /* this would display xml file help on a read-only wndproperty if
        it exists */
     return OnHelp();
-  } else if (mDataField != NULL && mDataField->supports_combolist) {
-    /* if this asserton fails, then there no valid root window could
-       be found - maybe it didn't register its wndproc? */
-    dlgComboPicker(this);
-    return true;
-  } else if (mDataField != NULL &&
-             mDataField->GetType() == DataField::Type::ROUGH_TIME) {
-    RoughTimeDataField &df = *(RoughTimeDataField *)mDataField;
-    RoughTime value = df.GetValue();
-    if (!TimeEntryDialog(GetCaption(), value, df.GetTimeZone(), true))
-      return true;
+  } else if (mDataField != NULL) {
+    if (!EditDataFieldDialog(GetCaption(), *mDataField, GetHelpText()))
+      return false;
 
-    df.ModifyValue(value);
-    RefreshDisplay();
-    return true;
-  } else if (mDataField != NULL &&
-             mDataField->GetType() == DataField::Type::GEOPOINT) {
-    GeoPointDataField &df = *(GeoPointDataField *)mDataField;
-    GeoPoint value = df.GetValue();
-    if (!GeoPointEntryDialog(GetCaption(), value, false))
-      return true;
-
-    df.ModifyValue(value);
     RefreshDisplay();
     return true;
   } else if (CanEditInPlace()) {
     // TODO: implement
     return true;
-  } else if (mDataField != NULL) {
-    const TCHAR *value = mDataField->GetAsString();
-    if (value == NULL)
-      return false;
-
-    StaticString<EDITSTRINGSIZE> buffer(value);
-
-    PrefixDataField::AllowedCharactersFunction acf;
-    if (mDataField->GetType() == DataField::Type::PREFIX)
-      acf = ((PrefixDataField *)mDataField)->GetAllowedCharactersFunction();
-
-    if (!TextEntryDialog(buffer, GetCaption(), acf))
-      return true;
-
-    mDataField->SetAsString(buffer);
-    RefreshDisplay();
-    return true;
   } else
     return false;
 }
@@ -241,9 +196,10 @@ WndProperty::UpdateLayout()
     edit_rc.right -= (DEFAULTBORDERPENWIDTH + 1);
     edit_rc.bottom -= (DEFAULTBORDERPENWIDTH + 1);
   } else {
+    const unsigned caption_height = look.text_font->GetHeight();
+
     edit_rc.left += (DEFAULTBORDERPENWIDTH + 1);
-    edit_rc.top = (edit_rc.top + edit_rc.bottom) / 2
-      - 2 * (DEFAULTBORDERPENWIDTH + 1);
+    edit_rc.top = DEFAULTBORDERPENWIDTH + caption_height;
     edit_rc.right -= (DEFAULTBORDERPENWIDTH + 1);
     edit_rc.bottom -= (DEFAULTBORDERPENWIDTH + 1);
   }
@@ -353,7 +309,7 @@ WndProperty::DecValue()
 void
 WndProperty::OnPaint(Canvas &canvas)
 {
-  const bool focused = HasFocus();
+  const bool focused = HasCursorKeys() && HasFocus();
 
   /* background and selector */
   if (pressed) {
@@ -384,7 +340,7 @@ WndProperty::OnPaint(Canvas &canvas)
       org.x = edit_rc.left;
       org.y = edit_rc.top - tsize.cy;
     } else {
-      org.x = caption_width - tsize.cx - Layout::FastScale(3);
+      org.x = caption_width - tsize.cx;
       org.y = (GetHeight() - tsize.cy) / 2;
     }
 
