@@ -24,7 +24,6 @@ Copyright_License {
 #include "MainWindow.hpp"
 #include "Startup.hpp"
 #include "MapWindow/GlueMapWindow.hpp"
-#include "resource.h"
 #include "InfoBoxes/InfoBoxManager.hpp"
 #include "InfoBoxes/InfoBoxLayout.hpp"
 #include "Interface.hpp"
@@ -34,7 +33,6 @@ Copyright_License {
 #include "Input/InputEvents.hpp"
 #include "Menu/ButtonLabel.hpp"
 #include "Screen/Layout.hpp"
-#include "Screen/Blank.hpp"
 #include "Dialogs/Airspace/AirspaceWarningDialog.hpp"
 #include "Audio/Sound.hpp"
 #include "Components.hpp"
@@ -47,7 +45,9 @@ Copyright_License {
 #include "Form/Form.hpp"
 #include "Widget/Widget.hpp"
 #include "UtilsSystem.hpp"
-#include "Look/Fonts.hpp"
+#include "Look/GlobalFonts.hpp"
+#include "Look/CustomFonts.hpp"
+#include "Look/DefaultFonts.hpp"
 #include "Look/Look.hpp"
 #include "Profile/ProfileKeys.hpp"
 #include "Profile/Profile.hpp"
@@ -61,8 +61,8 @@ Copyright_License {
 #include "Dialogs/Message.hpp"
 #endif
 
-#ifdef ENABLE_OPENGL
-#include "Screen/OpenGL/Cache.hpp"
+#ifdef HAVE_TEXT_CACHE
+#include "Screen/Custom/Cache.hpp"
 #endif
 
 #if !defined(WIN32) && !defined(ANDROID)
@@ -73,6 +73,8 @@ Copyright_License {
 #include <unistd.h> /* for execl() */
 #endif
 
+static constexpr unsigned separator_height = 2;
+
 gcc_pure
 static PixelRect
 GetBottomWidgetRect(const PixelRect &rc, const Widget *bottom_widget)
@@ -106,6 +108,8 @@ GetMapRectAbove(const PixelRect &rc, const PixelRect &bottom_rect)
 {
   PixelRect result = rc;
   result.bottom = bottom_rect.top;
+  if (bottom_rect.top < bottom_rect.bottom)
+    result.bottom -= separator_height;
   return result;
 }
 
@@ -145,9 +149,13 @@ MainWindow::Create(PixelSize size, TopWindowStyle style)
 
 gcc_noreturn
 static void
-NoFontsAvailable()
+FatalError(const TCHAR *msg)
 {
-  const TCHAR *msg = _T("Font initialisation failed");
+#if defined(HAVE_POSIX) && defined(NDEBUG)
+  /* make sure this gets written to stderr in any case; LogFormat()
+     will write to stderr only in debug builds */
+  fprintf(stderr, "%s\n", msg);
+#endif
 
   /* log the error */
   LogFormat(_T("%s"), msg);
@@ -155,13 +163,20 @@ NoFontsAvailable()
   /* now try to get a GUI error message out to the user */
 #ifdef WIN32
   MessageBox(NULL, msg, _T("XCSoar"), MB_ICONEXCLAMATION|MB_OK);
-#elif !defined(ANDROID)
+#elif !defined(ANDROID) && !defined(KOBO)
   execl("/usr/bin/xmessage", "xmessage", msg, NULL);
   execl("/usr/X11/bin/xmessage", "xmessage", msg, NULL);
 #endif
   exit(EXIT_FAILURE);
 }
 
+gcc_noreturn
+static void
+NoFontsAvailable()
+{
+  FatalError(_T("Font initialisation failed"));
+}
+
 void
 MainWindow::Initialise()
 {
@@ -176,7 +191,8 @@ MainWindow::Initialise()
   if (look == NULL)
     look = new Look();
 
-  look->Initialise(Fonts::map, Fonts::map_bold, Fonts::map_label);
+  look->Initialise(Fonts::dialog, Fonts::dialog_bold, Fonts::dialog_small,
+                   Fonts::map);
 }
 
 void
@@ -189,8 +205,7 @@ MainWindow::InitialiseConfigured()
   const InfoBoxLayout::Layout ib_layout =
     InfoBoxLayout::Calculate(rc, ui_settings.info_boxes.geometry);
 
-  Fonts::SizeInfoboxFont(ib_layout.control_width);
-
+#ifndef GNAV
   if (ui_settings.custom_fonts) {
     LogFormat("Load custom fonts");
     if (!Fonts::LoadCustom()) {
@@ -201,15 +216,20 @@ MainWindow::InitialiseConfigured()
       }
     }
 
-#ifdef ENABLE_OPENGL
+#ifdef HAVE_TEXT_CACHE
     /* fonts may have changed, discard all pre-rendered font
        textures */
     TextCache::Flush();
 #endif
   }
+#endif
+
+  Fonts::SizeInfoboxFont(ib_layout.control_size.cx);
 
   assert(look != NULL);
   look->InitialiseConfigured(CommonInterface::GetUISettings(),
+                             Fonts::dialog, Fonts::dialog_bold,
+                             Fonts::dialog_small,
                              Fonts::map, Fonts::map_bold, Fonts::map_label,
                              Fonts::cdi, Fonts::monospace,
                              Fonts::infobox, Fonts::infobox_small,
@@ -222,7 +242,7 @@ MainWindow::InitialiseConfigured()
   map_rect = ib_layout.remaining;
 
   ButtonLabel::CreateButtonLabels(*this);
-  ButtonLabel::SetFont(Fonts::map_bold);
+  ButtonLabel::SetFont(Fonts::dialog_bold);
 
   ReinitialiseLayout_vario(ib_layout);
 
@@ -288,8 +308,8 @@ void
 MainWindow::ReinitialiseLayoutTA(PixelRect rc,
                                  const InfoBoxLayout::Layout &layout)
 {
-  UPixelScalar sz = std::min(layout.control_height,
-                             layout.control_width) * 2;
+  UPixelScalar sz = std::min(layout.control_size.cy,
+                             layout.control_size.cx) * 2;
   rc.right = rc.left + sz;
   rc.top = rc.bottom - sz;
   thermal_assistant.Move(rc);
@@ -298,16 +318,13 @@ MainWindow::ReinitialiseLayoutTA(PixelRect rc,
 void
 MainWindow::ReinitialiseLayout()
 {
-  if (map == NULL) {
-#ifdef ANDROID
-    if (HasDialog())
-      dialogs.top()->ReinitialiseLayout();  // adapt simulator prompt
-#endif
+  if (map == nullptr)
     /* without the MapWindow, it is safe to assume that the MainWindow
        is just being initialized, and the InfoBoxes aren't initialized
        yet either, so there is nothing to do here */
     return;
-  }
+
+  const PixelRect rc = GetClientRect();
 
 #ifndef ENABLE_OPENGL
   if (draw_thread == NULL)
@@ -319,11 +336,10 @@ MainWindow::ReinitialiseLayout()
 
   const UISettings &ui_settings = CommonInterface::GetUISettings();
 
-  const PixelRect rc = GetClientRect();
   const InfoBoxLayout::Layout ib_layout =
     InfoBoxLayout::Calculate(rc, ui_settings.info_boxes.geometry);
 
-  Fonts::SizeInfoboxFont(ib_layout.control_width);
+  Fonts::SizeInfoboxFont(ib_layout.control_size.cx);
 
   InfoBoxManager::Create(*this, ib_layout, look->info_box, look->units);
   InfoBoxManager::ProcessTimer();
@@ -358,12 +374,6 @@ MainWindow::ReinitialiseLayout()
   if (widget != NULL)
     widget->Move(GetMainRect(rc));
 
-#ifdef ANDROID
-  // move topmost dialog to fit into the current layout, or close it
-  if (HasDialog())
-    dialogs.top()->ReinitialiseLayout();
-#endif
-
   if (map != NULL)
     map->BringToBottom();
 }
@@ -377,15 +387,14 @@ MainWindow::ReinitialiseLayout_flarm(PixelRect rc, const InfoBoxLayout::Layout i
   // Automatic mode - follow info boxes
   if (val == TrafficSettings::GaugeLocation::Auto) {
     switch (InfoBoxManager::layout.geometry) {
-    case InfoBoxSettings::Geometry::TOP_8:
-      val = TrafficSettings::GaugeLocation::TopRight;
-      break;
-    case InfoBoxSettings::Geometry::LEFT_8:
-      val = TrafficSettings::GaugeLocation::BottomLeft;
-      break;
-    case InfoBoxSettings::Geometry::TOP_12:
-      val = TrafficSettings::GaugeLocation::TopLeft;
+    case InfoBoxSettings::Geometry::TOP_LEFT_8:
+    case InfoBoxSettings::Geometry::TOP_LEFT_12:
+      if (InfoBoxManager::layout.landscape)
+        val = TrafficSettings::GaugeLocation::BottomLeft;
+      else
+        val = TrafficSettings::GaugeLocation::TopRight;
       break;
+
     default:
       val = TrafficSettings::GaugeLocation::BottomRight;    // Assume bottom right unles...
       break;
@@ -394,40 +403,40 @@ MainWindow::ReinitialiseLayout_flarm(PixelRect rc, const InfoBoxLayout::Layout i
 
   switch (val) {
   case TrafficSettings::GaugeLocation::TopLeft:
-    rc.right = rc.left + ib_layout.control_width * 2;
+    rc.right = rc.left + ib_layout.control_size.cx * 2;
     ++rc.left;
-    rc.bottom = rc.top + ib_layout.control_height * 2;
+    rc.bottom = rc.top + ib_layout.control_size.cy * 2;
     ++rc.top;
     break;
 
   case TrafficSettings::GaugeLocation::TopRight:
-    rc.left = rc.right - ib_layout.control_width * 2 + 1;
-    rc.bottom = rc.top + ib_layout.control_height * 2;
+    rc.left = rc.right - ib_layout.control_size.cx * 2 + 1;
+    rc.bottom = rc.top + ib_layout.control_size.cy * 2;
     ++rc.top;
     break;
 
   case TrafficSettings::GaugeLocation::BottomLeft:
-    rc.right = rc.left + ib_layout.control_width * 2;
+    rc.right = rc.left + ib_layout.control_size.cx * 2;
     ++rc.left;
-    rc.top = rc.bottom - ib_layout.control_height * 2 + 1;
+    rc.top = rc.bottom - ib_layout.control_size.cy * 2 + 1;
     break;
 
   case TrafficSettings::GaugeLocation::CentreTop:
-    rc.left = (rc.left + rc.right) / 2 - ib_layout.control_width;
-    rc.right = rc.left + ib_layout.control_width * 2 - 1;
-    rc.bottom = rc.top + ib_layout.control_height * 2;
+    rc.left = (rc.left + rc.right) / 2 - ib_layout.control_size.cx;
+    rc.right = rc.left + ib_layout.control_size.cx * 2 - 1;
+    rc.bottom = rc.top + ib_layout.control_size.cy * 2;
     ++rc.top;
     break;
 
   case TrafficSettings::GaugeLocation::CentreBottom:
-    rc.left = (rc.left + rc.right) / 2 - ib_layout.control_width;
-    rc.right = rc.left + ib_layout.control_width * 2 - 1;
-    rc.top = rc.bottom - ib_layout.control_height * 2 + 1;
+    rc.left = (rc.left + rc.right) / 2 - ib_layout.control_size.cx;
+    rc.right = rc.left + ib_layout.control_size.cx * 2 - 1;
+    rc.top = rc.bottom - ib_layout.control_size.cy * 2 + 1;
     break;
 
   default:    // aka flBottomRight
-    rc.left = rc.right - ib_layout.control_width * 2 + 1;
-    rc.top = rc.bottom - ib_layout.control_height * 2 + 1;
+    rc.left = rc.right - ib_layout.control_size.cx * 2 + 1;
+    rc.top = rc.bottom - ib_layout.control_size.cy * 2 + 1;
     break;
   }
 
@@ -454,6 +463,8 @@ void
 MainWindow::BeginShutdown()
 {
   timer.Cancel();
+
+  KillBottomWidget();
 }
 
 void
@@ -479,6 +490,13 @@ MainWindow::SetDefaultFocus()
     SetFocus();
 }
 
+void
+MainWindow::FlushRendererCaches()
+{
+  if (map != nullptr)
+    map->FlushCaches();
+}
+
 void
 MainWindow::FullRedraw()
 {
@@ -491,10 +509,10 @@ MainWindow::FullRedraw()
 void
 MainWindow::OnResize(PixelSize new_size)
 {
-  SingleWindow::OnResize(new_size);
-
   Layout::Initialize(new_size);
 
+  SingleWindow::OnResize(new_size);
+
   ReinitialiseLayout();
 
   if (map != NULL) {
@@ -614,7 +632,9 @@ MainWindow::OnMouseMove(PixelScalar x, PixelScalar y, unsigned keys)
 bool
 MainWindow::OnKeyDown(unsigned key_code)
 {
-  return InputEvents::processKey(key_code) ||
+  return (widget != nullptr && widget->KeyPress(key_code)) ||
+    (bottom_widget != nullptr && bottom_widget->KeyPress(key_code)) ||
+    InputEvents::processKey(key_code) ||
     SingleWindow::OnKeyDown(key_code);
 }
 
@@ -726,6 +746,20 @@ bool MainWindow::OnClose() {
   return true;
 }
 
+void
+MainWindow::OnPaint(Canvas &canvas)
+{
+  if (bottom_widget != nullptr && map != nullptr) {
+    /* draw a separator between main area and bottom area */
+    PixelRect rc = map->GetPosition();
+    rc.top = rc.bottom;
+    rc.bottom += separator_height;
+    canvas.DrawFilledRectangle(rc, COLOR_BLACK);
+  }
+
+  SingleWindow::OnPaint(canvas);
+}
+
 void
 MainWindow::SetFullScreen(bool _full_screen)
 {
