@@ -23,7 +23,6 @@ Copyright_License {
 
 #include "Screen/Timer.hpp"
 #include "Screen/Window.hpp"
-#include "Event/Timer.hpp"
 
 void
 WindowTimer::Schedule(unsigned ms)
@@ -40,12 +39,3 @@ WindowTimer::Cancel()
   ::KillTimer(window, (UINT_PTR)this);
   id = 0;
 }
-
-bool
-Timer::OnTimer(WindowTimer &timer)
-{
-  assert(timer == *(WindowTimer *)this);
-
-  OnTimer();
-  return true;
-}
