@@ -28,6 +28,18 @@ Copyright_License {
 #include "Texture.hpp"
 #include "Debug.hpp"
 
+void
+Bitmap::EnableInterpolation()
+{
+  interpolation = true;
+  if (texture != nullptr) {
+    texture->Bind();
+    texture->EnableInterpolation();
+  }
+}
+
+#ifndef ANDROID
+
 bool
 Bitmap::Load(const UncompressedImage &uncompressed, gcc_unused Type type)
 {
@@ -36,23 +48,13 @@ Bitmap::Load(const UncompressedImage &uncompressed, gcc_unused Type type)
   if (texture == nullptr)
     return false;
 
+  if (interpolation)
+    texture->EnableInterpolation();
+
   size = { uncompressed.GetWidth(), uncompressed.GetHeight() };
   return true;
 }
 
-#ifndef ANDROID
-
-bool
-Bitmap::Reload()
-{
-  assert(id != 0);
-  assert(texture == NULL);
-
-  /* XXX this is no real implementation; we currently support OpenGL
-     surface reinitialisation only on Android */
-  return Load(id);
-}
-
 void
 Bitmap::Reset()
 {
@@ -65,19 +67,6 @@ Bitmap::Reset()
 
 #endif /* !ANDROID */
 
-void
-Bitmap::SurfaceCreated()
-{
-  Reload();
-}
-
-void
-Bitmap::SurfaceDestroyed()
-{
-  delete texture;
-  texture = NULL;
-}
-
 const PixelSize
 Bitmap::GetSize() const
 {
