@@ -24,27 +24,32 @@ Copyright_License {
 #include "Topography/TopographyFileRenderer.hpp"
 #include "Topography/TopographyFile.hpp"
 #include "Topography/XShape.hpp"
-#include "Look/Fonts.hpp"
+#include "Look/GlobalFonts.hpp"
 #include "Renderer/LabelBlock.hpp"
 #include "Projection/WindowProjection.hpp"
-#include "resource.h"
 #include "Screen/Canvas.hpp"
 #include "Screen/Features.hpp"
 #include "Screen/Layout.hpp"
 #include "Math/Matrix2D.hpp"
 #include "shapelib/mapserver.h"
 #include "Util/AllocatedArray.hpp"
+#include "Util/tstring.hpp"
 #include "Geo/GeoClip.hpp"
 
+#ifdef ENABLE_OPENGL
+#include "Screen/OpenGL/Scope.hpp"
+#endif
+
 #include <algorithm>
+#include <set>
 
 TopographyFileRenderer::TopographyFileRenderer(const TopographyFile &_file)
   :file(_file), pen(file.GetPenWidth(), file.GetColor()),
    brush(file.GetColor())
 {
-  int icon_ID = file.GetIcon();
-  if (icon_ID != 0)
-    icon.Load(icon_ID, 5000+icon_ID);
+  ResourceId icon_ID = file.GetIcon();
+  if (icon_ID.IsDefined())
+    icon.LoadResource(icon_ID, file.GetBigIcon());
 }
 
 void
@@ -284,8 +289,13 @@ TopographyFileRenderer::Paint(Canvas &canvas,
 #else
         glVertexPointer(2, GL_INT, 0, &points[0].x);
 #endif
-        glDrawElements(GL_TRIANGLE_STRIP, *index_count, GL_UNSIGNED_SHORT,
-                       triangles);
+        if (!brush.GetColor().IsOpaque()) {
+          const GLBlend blend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+          glDrawElements(GL_TRIANGLE_STRIP, *index_count, GL_UNSIGNED_SHORT,
+                         triangles);
+        } else
+          glDrawElements(GL_TRIANGLE_STRIP, *index_count, GL_UNSIGNED_SHORT,
+                         triangles);
       }
 #else // !ENABLE_OPENGL
       for (; lines < end_lines; ++lines) {
@@ -368,6 +378,8 @@ TopographyFileRenderer::PaintLabels(Canvas &canvas,
   m1.Scale(projection.GetScale());
 #endif
 
+  std::set<tstring> drawn_labels;
+
   // Iterate over all shapes in the file
   for (auto it = visible_labels.begin(), end = visible_labels.end();
        it != end; ++it) {
@@ -429,6 +441,9 @@ TopographyFileRenderer::PaintLabels(Canvas &canvas,
       if (!label_block.check(brect))
         continue;
 
+      if (!drawn_labels.insert(label).second)
+        continue;
+
       canvas.DrawText(minx, miny, label);
     }
   }
